@@ -1,7 +1,15 @@
+"""
+This is where the mainline sits and is responsible for setting up the logging,
+the argument parsing and for starting up Harpoon.
+"""
+
+from __future__ import print_function
+
 from harpoon.errors import BadOption, BadDockerConnection
-from harpoon.overview import Harpoon
+from harpoon.overview import Overview
 
 from rainbow_logging_handler import RainbowLoggingHandler
+from input_algorithms.spec_base import NotSpecified
 from docker.client import Client as DockerClient
 from delfick_error import DelfickError
 import requests
@@ -14,10 +22,7 @@ import os
 
 log = logging.getLogger("harpoon.executor")
 
-class NotSpecified(object):
-    """Tell the difference between None and Empty"""
-
-def setup_logging(verbose=False, silent=False):
+def setup_logging(verbose=False, silent=False, debug=False):
     log = logging.getLogger("")
     handler = RainbowLoggingHandler(sys.stderr)
     handler._column_color['%(asctime)s'] = ('cyan', None, False)
@@ -25,11 +30,11 @@ def setup_logging(verbose=False, silent=False):
     handler._column_color['%(message)s'][logging.INFO] = ('blue', None, False)
     handler.setFormatter(logging.Formatter("%(asctime)s %(levelname)-7s %(name)-15s %(message)s"))
     log.addHandler(handler)
-    log.setLevel([logging.INFO, logging.DEBUG][verbose])
+    log.setLevel([logging.INFO, logging.DEBUG][verbose or debug])
     if silent:
         log.setLevel(logging.ERROR)
 
-    logging.getLogger("requests").setLevel([logging.CRITICAL, logging.ERROR][verbose])
+    logging.getLogger("requests").setLevel([logging.CRITICAL, logging.ERROR][verbose or debug])
     return handler
 
 class CliParser(object):
@@ -90,6 +95,11 @@ class CliParser(object):
             , action = "store_true"
             )
 
+        logging.add_argument("--debug"
+            , help = "Debug logs"
+            , action = "store_true"
+            )
+
         opts = {}
         if os.path.exists("./harpoon.yml"):
             opts["default"] = "./harpoon.yml"
@@ -121,7 +131,7 @@ class CliParser(object):
             , action = "store_false"
             )
 
-        extra = {}
+        extra = {"default": ""}
         if default_image is not NotSpecified:
             extra["default"] = default_image
         parser.add_argument("--image"
@@ -214,7 +224,7 @@ def docker_context():
 def main(argv=None):
     try:
         args, extra = CliParser().parse_args(argv)
-        handler = setup_logging(verbose=args.verbose, silent=args.silent)
+        handler = setup_logging(verbose=args.verbose, silent=args.silent, debug=args.debug)
 
         cli_args = {"harpoon": {}}
         for key, val in sorted(vars(args).items()):
@@ -223,13 +233,20 @@ def main(argv=None):
             else:
                 cli_args[key] = val
         cli_args["harpoon"]["extra"] = extra
+        cli_args["harpoon"]["docker_context"] = docker_context()
 
-        Harpoon(configuration_file=args.harpoon_config.name, docker_context=docker_context(), logging_handler=handler).start(cli_args)
+        for key in ('bash', 'command'):
+            if cli_args[key] is None:
+                cli_args[key] = NotSpecified
+
+        Overview(configuration_file=args.harpoon_config.name, logging_handler=handler).start(cli_args)
     except DelfickError as error:
-        print ""
-        print "!" * 80
-        print "Something went wrong! -- {0}".format(error.__class__.__name__)
-        print "\t{0}".format(error)
+        print("")
+        print("!" * 80)
+        print("Something went wrong! -- {0}".format(error.__class__.__name__))
+        print("\t{0}".format(error))
+        if args.debug:
+            raise
         sys.exit(1)
 
 if __name__ == '__main__':
